/********************************************************************************
** Form generated from reading UI file 'welcomepage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOMEPAGE_H
#define UI_WELCOMEPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <fcitxqti18nhelper.h>
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_WelcomePage
{
public:
    QVBoxLayout *verticalLayout;
    KMessageWidget *fcitxNotRunningMessage;
    QLabel *label;
    QSpacerItem *verticalSpacer;

    void setupUi(QWizardPage *WelcomePage)
    {
        if (WelcomePage->objectName().isEmpty())
            WelcomePage->setObjectName("WelcomePage");
        WelcomePage->resize(400, 300);
        verticalLayout = new QVBoxLayout(WelcomePage);
        verticalLayout->setObjectName("verticalLayout");
        fcitxNotRunningMessage = new KMessageWidget(WelcomePage);
        fcitxNotRunningMessage->setObjectName("fcitxNotRunningMessage");
        fcitxNotRunningMessage->setMessageType(KMessageWidget::Error);

        verticalLayout->addWidget(fcitxNotRunningMessage);

        label = new QLabel(WelcomePage);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(WelcomePage);

        QMetaObject::connectSlotsByName(WelcomePage);
    } // setupUi

    void retranslateUi(QWizardPage *WelcomePage)
    {
        WelcomePage->setWindowTitle(fcitx::tr2fcitx("WelcomePage", nullptr));
        WelcomePage->setTitle(fcitx::tr2fcitx("Migrate Data from Fcitx 4", nullptr));
        fcitxNotRunningMessage->setText(fcitx::tr2fcitx("Fcitx 5 is not running.", nullptr));
        label->setText(fcitx::tr2fcitx("This wizard will help you to migrate existing user data from Fcitx 4 to Fcitx 5", nullptr));
    } // retranslateUi

};

namespace Ui {
    class WelcomePage: public Ui_WelcomePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WELCOMEPAGE_H
