/****************************************************************************
** Meta object code from reading C++ file 'skkmigrator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../fcitx5-configtool/src/migrator/skk/skkmigrator.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'skkmigrator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5fcitx11SkkMigratorE_t {};
} // unnamed namespace

template <> constexpr inline auto fcitx::SkkMigrator::qt_create_metaobjectdata<qt_meta_tag_ZN5fcitx11SkkMigratorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "fcitx::SkkMigrator"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SkkMigrator, qt_meta_tag_ZN5fcitx11SkkMigratorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject fcitx::SkkMigrator::staticMetaObject = { {
    QMetaObject::SuperData::link<Migrator::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx11SkkMigratorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx11SkkMigratorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5fcitx11SkkMigratorE_t>.metaTypes,
    nullptr
} };

void fcitx::SkkMigrator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SkkMigrator *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *fcitx::SkkMigrator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *fcitx::SkkMigrator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx11SkkMigratorE_t>.strings))
        return static_cast<void*>(this);
    return Migrator::qt_metacast(_clname);
}

int fcitx::SkkMigrator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Migrator::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto fcitx::SkkMigratorPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "fcitx::SkkMigratorPlugin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SkkMigratorPlugin, qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject fcitx::SkkMigratorPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<FcitxMigratorFactoryPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>.metaTypes,
    nullptr
} };

void fcitx::SkkMigratorPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SkkMigratorPlugin *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *fcitx::SkkMigratorPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *fcitx::SkkMigratorPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx17SkkMigratorPluginE_t>.strings))
        return static_cast<void*>(this);
    return FcitxMigratorFactoryPlugin::qt_metacast(_clname);
}

int fcitx::SkkMigratorPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FcitxMigratorFactoryPlugin::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace fcitx;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_SkkMigratorPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x2d,  'o',  'r',  'g',  '.',  'f', 
    'c',  'i',  't',  'x',  '.',  'F',  'c',  'i', 
    't',  'x',  '.',  'F',  'c',  'i',  't',  'x', 
    'M',  'i',  'g',  'r',  'a',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    // "className"
    0x03,  0x71,  'S',  'k',  'k',  'M',  'i',  'g', 
    'r',  'a',  't',  'o',  'r',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa1,  0x65,  'a',  'd',  'd',  'o',  'n', 
    0x63,  's',  'k',  'k', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(fcitx::SkkMigratorPlugin, SkkMigratorPlugin, qt_pluginMetaDataV2_SkkMigratorPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_SkkMigratorPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x2d,  'o',  'r',  'g',  '.',  'f', 
    'c',  'i',  't',  'x',  '.',  'F',  'c',  'i', 
    't',  'x',  '.',  'F',  'c',  'i',  't',  'x', 
    'M',  'i',  'g',  'r',  'a',  't',  'o',  'r', 
    'F',  'a',  'c',  't',  'o',  'r',  'y',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    // "className"
    0x03,  0x71,  'S',  'k',  'k',  'M',  'i',  'g', 
    'r',  'a',  't',  'o',  'r',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa1,  0x65,  'a',  'd',  'd',  'o',  'n', 
    0x63,  's',  'k',  'k', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(fcitx::SkkMigratorPlugin, SkkMigratorPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
