/********************************************************************************
** Form generated from reading UI file 'layoutselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAYOUTSELECTOR_H
#define UI_LAYOUTSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_LayoutSelector
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label_3;
    QComboBox *languageComboBox;
    QLabel *label;
    QComboBox *layoutComboBox;
    QLabel *label_2;
    QComboBox *variantComboBox;

    void setupUi(QWidget *LayoutSelector)
    {
        if (LayoutSelector->objectName().isEmpty())
            LayoutSelector->setObjectName("LayoutSelector");
        LayoutSelector->resize(600, 300);
        LayoutSelector->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(LayoutSelector);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label_3 = new QLabel(LayoutSelector);
        label_3->setObjectName("label_3");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_3);

        languageComboBox = new QComboBox(LayoutSelector);
        languageComboBox->setObjectName("languageComboBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(languageComboBox->sizePolicy().hasHeightForWidth());
        languageComboBox->setSizePolicy(sizePolicy);
        languageComboBox->setMinimumSize(QSize(400, 0));

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, languageComboBox);

        label = new QLabel(LayoutSelector);
        label->setObjectName("label");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        layoutComboBox = new QComboBox(LayoutSelector);
        layoutComboBox->setObjectName("layoutComboBox");
        sizePolicy.setHeightForWidth(layoutComboBox->sizePolicy().hasHeightForWidth());
        layoutComboBox->setSizePolicy(sizePolicy);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, layoutComboBox);

        label_2 = new QLabel(LayoutSelector);
        label_2->setObjectName("label_2");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_2);

        variantComboBox = new QComboBox(LayoutSelector);
        variantComboBox->setObjectName("variantComboBox");
        sizePolicy.setHeightForWidth(variantComboBox->sizePolicy().hasHeightForWidth());
        variantComboBox->setSizePolicy(sizePolicy);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, variantComboBox);


        verticalLayout->addLayout(formLayout);


        retranslateUi(LayoutSelector);

        QMetaObject::connectSlotsByName(LayoutSelector);
    } // setupUi

    void retranslateUi(QWidget *LayoutSelector)
    {
        label_3->setText(fcitx::tr2fcitx("Language:", nullptr));
        label->setText(fcitx::tr2fcitx("Layout:", nullptr));
        label_2->setText(fcitx::tr2fcitx("Variant:", nullptr));
        (void)LayoutSelector;
    } // retranslateUi

};

namespace Ui {
    class LayoutSelector: public Ui_LayoutSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LAYOUTSELECTOR_H
