/********************************************************************************
** Form generated from reading UI file 'impage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPAGE_H
#define UI_IMPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_IMPage
{
public:
    QVBoxLayout *verticalLayout_4;
    KMessageWidget *checkUpdateMessage;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label_5;
    QComboBox *inputMethodGroupComboBox;
    QToolButton *addGroupButton;
    QToolButton *deleteGroupButton;
    QListView *currentIMView;
    QPushButton *defaultLayoutButton;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer;
    QToolButton *addIMButton;
    QToolButton *removeIMButton;
    QToolButton *moveUpButton;
    QToolButton *moveDownButton;
    QToolButton *configureButton;
    QToolButton *layoutButton;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QLineEdit *filterTextEdit;
    QTreeView *availIMView;
    QCheckBox *onlyCurrentLanguageCheckBox;
    QFrame *line;
    QHBoxLayout *horizontalLayout_3;
    QLabel *infoIconLabel;
    QLabel *infoLabel;

    void setupUi(QWidget *IMPage)
    {
        if (IMPage->objectName().isEmpty())
            IMPage->setObjectName("IMPage");
        IMPage->resize(752, 469);
        verticalLayout_4 = new QVBoxLayout(IMPage);
        verticalLayout_4->setObjectName("verticalLayout_4");
        checkUpdateMessage = new KMessageWidget(IMPage);
        checkUpdateMessage->setObjectName("checkUpdateMessage");
        checkUpdateMessage->setWordWrap(true);

        verticalLayout_4->addWidget(checkUpdateMessage);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label_2 = new QLabel(IMPage);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_5 = new QLabel(IMPage);
        label_5->setObjectName("label_5");

        horizontalLayout->addWidget(label_5);

        inputMethodGroupComboBox = new QComboBox(IMPage);
        inputMethodGroupComboBox->setObjectName("inputMethodGroupComboBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(inputMethodGroupComboBox->sizePolicy().hasHeightForWidth());
        inputMethodGroupComboBox->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(inputMethodGroupComboBox);

        addGroupButton = new QToolButton(IMPage);
        addGroupButton->setObjectName("addGroupButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(addGroupButton->sizePolicy().hasHeightForWidth());
        addGroupButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(addGroupButton);

        deleteGroupButton = new QToolButton(IMPage);
        deleteGroupButton->setObjectName("deleteGroupButton");
        sizePolicy1.setHeightForWidth(deleteGroupButton->sizePolicy().hasHeightForWidth());
        deleteGroupButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(deleteGroupButton);


        verticalLayout->addLayout(horizontalLayout);

        currentIMView = new QListView(IMPage);
        currentIMView->setObjectName("currentIMView");

        verticalLayout->addWidget(currentIMView);

        defaultLayoutButton = new QPushButton(IMPage);
        defaultLayoutButton->setObjectName("defaultLayoutButton");

        verticalLayout->addWidget(defaultLayoutButton);


        horizontalLayout_2->addLayout(verticalLayout);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        addIMButton = new QToolButton(IMPage);
        addIMButton->setObjectName("addIMButton");
        addIMButton->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(addIMButton->sizePolicy().hasHeightForWidth());
        addIMButton->setSizePolicy(sizePolicy2);
        addIMButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(addIMButton);

        removeIMButton = new QToolButton(IMPage);
        removeIMButton->setObjectName("removeIMButton");
        removeIMButton->setEnabled(false);
        sizePolicy2.setHeightForWidth(removeIMButton->sizePolicy().hasHeightForWidth());
        removeIMButton->setSizePolicy(sizePolicy2);
        removeIMButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(removeIMButton);

        moveUpButton = new QToolButton(IMPage);
        moveUpButton->setObjectName("moveUpButton");
        moveUpButton->setEnabled(false);
        sizePolicy2.setHeightForWidth(moveUpButton->sizePolicy().hasHeightForWidth());
        moveUpButton->setSizePolicy(sizePolicy2);
        moveUpButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(moveUpButton);

        moveDownButton = new QToolButton(IMPage);
        moveDownButton->setObjectName("moveDownButton");
        moveDownButton->setEnabled(false);
        sizePolicy2.setHeightForWidth(moveDownButton->sizePolicy().hasHeightForWidth());
        moveDownButton->setSizePolicy(sizePolicy2);
        moveDownButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(moveDownButton);

        configureButton = new QToolButton(IMPage);
        configureButton->setObjectName("configureButton");
        sizePolicy2.setHeightForWidth(configureButton->sizePolicy().hasHeightForWidth());
        configureButton->setSizePolicy(sizePolicy2);
        configureButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(configureButton);

        layoutButton = new QToolButton(IMPage);
        layoutButton->setObjectName("layoutButton");
        sizePolicy2.setHeightForWidth(layoutButton->sizePolicy().hasHeightForWidth());
        layoutButton->setSizePolicy(sizePolicy2);
        layoutButton->setMinimumSize(QSize(0, 0));

        verticalLayout_3->addWidget(layoutButton);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);


        horizontalLayout_2->addLayout(verticalLayout_3);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(IMPage);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        filterTextEdit = new QLineEdit(IMPage);
        filterTextEdit->setObjectName("filterTextEdit");
        filterTextEdit->setClearButtonEnabled(true);

        verticalLayout_2->addWidget(filterTextEdit);

        availIMView = new QTreeView(IMPage);
        availIMView->setObjectName("availIMView");
        availIMView->setIndentation(0);
        availIMView->setRootIsDecorated(false);
        availIMView->setItemsExpandable(false);
        availIMView->setSortingEnabled(true);
        availIMView->setHeaderHidden(true);
        availIMView->header()->setVisible(false);

        verticalLayout_2->addWidget(availIMView);

        onlyCurrentLanguageCheckBox = new QCheckBox(IMPage);
        onlyCurrentLanguageCheckBox->setObjectName("onlyCurrentLanguageCheckBox");
        onlyCurrentLanguageCheckBox->setChecked(true);
        onlyCurrentLanguageCheckBox->setTristate(false);

        verticalLayout_2->addWidget(onlyCurrentLanguageCheckBox);


        horizontalLayout_2->addLayout(verticalLayout_2);


        verticalLayout_4->addLayout(horizontalLayout_2);

        line = new QFrame(IMPage);
        line->setObjectName("line");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(line->sizePolicy().hasHeightForWidth());
        line->setSizePolicy(sizePolicy3);
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        infoIconLabel = new QLabel(IMPage);
        infoIconLabel->setObjectName("infoIconLabel");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(infoIconLabel->sizePolicy().hasHeightForWidth());
        infoIconLabel->setSizePolicy(sizePolicy4);
        infoIconLabel->setMinimumSize(QSize(22, 22));
        infoIconLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_3->addWidget(infoIconLabel);

        infoLabel = new QLabel(IMPage);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        infoLabel->setWordWrap(true);

        horizontalLayout_3->addWidget(infoLabel);


        verticalLayout_4->addLayout(horizontalLayout_3);


        retranslateUi(IMPage);

        QMetaObject::connectSlotsByName(IMPage);
    } // setupUi

    void retranslateUi(QWidget *IMPage)
    {
        IMPage->setWindowTitle(fcitx::tr2fcitx("Form", "KDE::DoNotExtract"));
        checkUpdateMessage->setText(fcitx::tr2fcitx("Found updates to fcitx installation. Do you want to check for newly installed input methods and addons? To update already loaded addons, fcitx would need to be restarted.", nullptr));
        label_2->setText(fcitx::tr2fcitx("Current Input Method:", nullptr));
        label_5->setText(fcitx::tr2fcitx("Group:", nullptr));
#if QT_CONFIG(tooltip)
        addGroupButton->setToolTip(fcitx::tr2fcitx("Add Group", nullptr));
#endif // QT_CONFIG(tooltip)
        addGroupButton->setText(fcitx::tr2fcitx("Add", nullptr));
#if QT_CONFIG(tooltip)
        deleteGroupButton->setToolTip(fcitx::tr2fcitx("Delete Group", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteGroupButton->setText(fcitx::tr2fcitx("Remove", nullptr));
#if QT_CONFIG(tooltip)
        defaultLayoutButton->setToolTip(fcitx::tr2fcitx("Keyboard layout to use when no input method active", nullptr));
#endif // QT_CONFIG(tooltip)
        defaultLayoutButton->setText(fcitx::tr2fcitx("Select system keyboard layout", nullptr));
#if QT_CONFIG(tooltip)
        addIMButton->setToolTip(fcitx::tr2fcitx("Add", nullptr));
#endif // QT_CONFIG(tooltip)
        addIMButton->setText(fcitx::tr2fcitx("Add", nullptr));
#if QT_CONFIG(tooltip)
        removeIMButton->setToolTip(fcitx::tr2fcitx("Remove", nullptr));
#endif // QT_CONFIG(tooltip)
        removeIMButton->setText(fcitx::tr2fcitx("Remove", nullptr));
#if QT_CONFIG(tooltip)
        moveUpButton->setToolTip(fcitx::tr2fcitx("Move Up", nullptr));
#endif // QT_CONFIG(tooltip)
        moveUpButton->setText(fcitx::tr2fcitx("Up", nullptr));
#if QT_CONFIG(tooltip)
        moveDownButton->setToolTip(fcitx::tr2fcitx("Move Down", nullptr));
#endif // QT_CONFIG(tooltip)
        moveDownButton->setText(fcitx::tr2fcitx("Down", nullptr));
#if QT_CONFIG(tooltip)
        configureButton->setToolTip(fcitx::tr2fcitx("Configure", nullptr));
#endif // QT_CONFIG(tooltip)
        configureButton->setText(fcitx::tr2fcitx("Configure", nullptr));
#if QT_CONFIG(tooltip)
        layoutButton->setToolTip(fcitx::tr2fcitx("Select Layout", nullptr));
#endif // QT_CONFIG(tooltip)
        layoutButton->setText(fcitx::tr2fcitx("Layout", nullptr));
        label->setText(fcitx::tr2fcitx("Available Input Method:", nullptr));
        filterTextEdit->setPlaceholderText(fcitx::tr2fcitx("Search Input Method", nullptr));
        onlyCurrentLanguageCheckBox->setText(fcitx::tr2fcitx("Only &Show Current Language", nullptr));
        infoIconLabel->setText(QString());
        infoLabel->setText(fcitx::tr2fcitx("The first input method will be inactive state. Usually you need to put <b>Keyboard</b> or <b>Keyboard - <i>layout name</i></b> in the first place.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class IMPage: public Ui_IMPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMPAGE_H
