/********************************************************************************
** Form generated from reading UI file 'erroroverlay.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ERROROVERLAY_H
#define UI_ERROROVERLAY_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_ErrorOverlay
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *pixmapLabel;
    QLabel *textLabel;
    QPushButton *runFcitxButton;
    QSpacerItem *horizontalSpacer_3;

    void setupUi(QWidget *ErrorOverlay)
    {
        if (ErrorOverlay->objectName().isEmpty())
            ErrorOverlay->setObjectName("ErrorOverlay");
        ErrorOverlay->resize(1471, 445);
        QPalette palette;
        QBrush brush(QColor(255, 255, 255, 255));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::WindowText, brush);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Text, brush);
        QBrush brush1(QColor(252, 252, 252, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Base, brush1);
        QBrush brush2(QColor(0, 0, 0, 128));
        brush2.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Window, brush2);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::WindowText, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Text, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Base, brush1);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Window, brush2);
        QBrush brush3(QColor(165, 167, 169, 255));
        brush3.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::WindowText, brush3);
        QBrush brush4(QColor(172, 174, 176, 255));
        brush4.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Text, brush4);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Base, brush2);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Window, brush2);
        ErrorOverlay->setPalette(palette);
        ErrorOverlay->setWindowTitle(QString::fromUtf8("Form"));
        ErrorOverlay->setAutoFillBackground(true);
        horizontalLayout = new QHBoxLayout(ErrorOverlay);
        horizontalLayout->setSpacing(10);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pixmapLabel = new QLabel(ErrorOverlay);
        pixmapLabel->setObjectName("pixmapLabel");
        pixmapLabel->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(pixmapLabel);

        textLabel = new QLabel(ErrorOverlay);
        textLabel->setObjectName("textLabel");
        textLabel->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(textLabel);

        runFcitxButton = new QPushButton(ErrorOverlay);
        runFcitxButton->setObjectName("runFcitxButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("system-run")));
        runFcitxButton->setIcon(icon);

        horizontalLayout->addWidget(runFcitxButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        retranslateUi(ErrorOverlay);

        QMetaObject::connectSlotsByName(ErrorOverlay);
    } // setupUi

    void retranslateUi(QWidget *ErrorOverlay)
    {
        pixmapLabel->setText(QString());
        textLabel->setText(fcitx::tr2fcitx("Cannot connect to Fcitx by DBus, is Fcitx running?", nullptr));
        runFcitxButton->setText(fcitx::tr2fcitx("Run Fcitx 5", nullptr));
        (void)ErrorOverlay;
    } // retranslateUi

};

namespace Ui {
    class ErrorOverlay: public Ui_ErrorOverlay {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ERROROVERLAY_H
