/********************************************************************************
** Form generated from reading UI file 'taskpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TASKPAGE_H
#define UI_TASKPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <fcitxqti18nhelper.h>
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_TaskPage
{
public:
    QVBoxLayout *verticalLayout;
    KMessageWidget *fcitxNotRunningMessage;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QListView *taskView;
    QCheckBox *selectAllBox;
    QLabel *descriptionLabel;

    void setupUi(QWizardPage *TaskPage)
    {
        if (TaskPage->objectName().isEmpty())
            TaskPage->setObjectName("TaskPage");
        TaskPage->resize(515, 407);
        verticalLayout = new QVBoxLayout(TaskPage);
        verticalLayout->setObjectName("verticalLayout");
        fcitxNotRunningMessage = new KMessageWidget(TaskPage);
        fcitxNotRunningMessage->setObjectName("fcitxNotRunningMessage");
        fcitxNotRunningMessage->setMessageType(KMessageWidget::Error);

        verticalLayout->addWidget(fcitxNotRunningMessage);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        taskView = new QListView(TaskPage);
        taskView->setObjectName("taskView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(taskView->sizePolicy().hasHeightForWidth());
        taskView->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(taskView);

        selectAllBox = new QCheckBox(TaskPage);
        selectAllBox->setObjectName("selectAllBox");

        verticalLayout_2->addWidget(selectAllBox);


        horizontalLayout->addLayout(verticalLayout_2);

        descriptionLabel = new QLabel(TaskPage);
        descriptionLabel->setObjectName("descriptionLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(descriptionLabel->sizePolicy().hasHeightForWidth());
        descriptionLabel->setSizePolicy(sizePolicy1);
        descriptionLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        descriptionLabel->setWordWrap(true);

        horizontalLayout->addWidget(descriptionLabel);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(TaskPage);

        QMetaObject::connectSlotsByName(TaskPage);
    } // setupUi

    void retranslateUi(QWizardPage *TaskPage)
    {
        TaskPage->setWindowTitle(QString());
        TaskPage->setTitle(fcitx::tr2fcitx("Select migration tasks", nullptr));
        fcitxNotRunningMessage->setText(fcitx::tr2fcitx("Fcitx 5 is not running.", nullptr));
        selectAllBox->setText(fcitx::tr2fcitx("Select &All", nullptr));
        descriptionLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class TaskPage: public Ui_TaskPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TASKPAGE_H
