/********************************************************************************
** Form generated from reading UI file 'listoptionwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LISTOPTIONWIDGET_H
#define UI_LISTOPTIONWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_ListOptionWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QListView *listView;
    QVBoxLayout *verticalLayout;
    QToolButton *addButton;
    QToolButton *editButton;
    QToolButton *moveUpButton;
    QToolButton *moveDownButton;
    QToolButton *removeButton;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ListOptionWidget)
    {
        if (ListOptionWidget->objectName().isEmpty())
            ListOptionWidget->setObjectName("ListOptionWidget");
        ListOptionWidget->resize(400, 300);
        ListOptionWidget->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(ListOptionWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        listView = new QListView(ListOptionWidget);
        listView->setObjectName("listView");

        horizontalLayout->addWidget(listView);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        addButton = new QToolButton(ListOptionWidget);
        addButton->setObjectName("addButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(addButton->sizePolicy().hasHeightForWidth());
        addButton->setSizePolicy(sizePolicy);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("list-add-symbolic")));
        addButton->setIcon(icon);

        verticalLayout->addWidget(addButton);

        editButton = new QToolButton(ListOptionWidget);
        editButton->setObjectName("editButton");
        sizePolicy.setHeightForWidth(editButton->sizePolicy().hasHeightForWidth());
        editButton->setSizePolicy(sizePolicy);
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("document-edit-symbolic")));
        editButton->setIcon(icon1);

        verticalLayout->addWidget(editButton);

        moveUpButton = new QToolButton(ListOptionWidget);
        moveUpButton->setObjectName("moveUpButton");
        sizePolicy.setHeightForWidth(moveUpButton->sizePolicy().hasHeightForWidth());
        moveUpButton->setSizePolicy(sizePolicy);
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("go-up-symbolic")));
        moveUpButton->setIcon(icon2);

        verticalLayout->addWidget(moveUpButton);

        moveDownButton = new QToolButton(ListOptionWidget);
        moveDownButton->setObjectName("moveDownButton");
        sizePolicy.setHeightForWidth(moveDownButton->sizePolicy().hasHeightForWidth());
        moveDownButton->setSizePolicy(sizePolicy);
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("go-down-symbolic")));
        moveDownButton->setIcon(icon3);

        verticalLayout->addWidget(moveDownButton);

        removeButton = new QToolButton(ListOptionWidget);
        removeButton->setObjectName("removeButton");
        sizePolicy.setHeightForWidth(removeButton->sizePolicy().hasHeightForWidth());
        removeButton->setSizePolicy(sizePolicy);
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("list-remove-symbolic")));
        removeButton->setIcon(icon4);

        verticalLayout->addWidget(removeButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout);


        retranslateUi(ListOptionWidget);

        QMetaObject::connectSlotsByName(ListOptionWidget);
    } // setupUi

    void retranslateUi(QWidget *ListOptionWidget)
    {
        addButton->setText(fcitx::tr2fcitx("Add", nullptr));
        editButton->setText(fcitx::tr2fcitx("Edit", nullptr));
        moveUpButton->setText(fcitx::tr2fcitx("Up", nullptr));
        moveDownButton->setText(fcitx::tr2fcitx("Down", nullptr));
        removeButton->setText(fcitx::tr2fcitx("Remove", nullptr));
        (void)ListOptionWidget;
    } // retranslateUi

};

namespace Ui {
    class ListOptionWidget: public Ui_ListOptionWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LISTOPTIONWIDGET_H
