/********************************************************************************
** Form generated from reading UI file 'progresspage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROGRESSPAGE_H
#define UI_PROGRESSPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_ProgressPage
{
public:
    QVBoxLayout *verticalLayout;
    QListWidget *listWidget;

    void setupUi(QWizardPage *ProgressPage)
    {
        if (ProgressPage->objectName().isEmpty())
            ProgressPage->setObjectName("ProgressPage");
        ProgressPage->resize(400, 300);
        verticalLayout = new QVBoxLayout(ProgressPage);
        verticalLayout->setObjectName("verticalLayout");
        listWidget = new QListWidget(ProgressPage);
        listWidget->setObjectName("listWidget");

        verticalLayout->addWidget(listWidget);


        retranslateUi(ProgressPage);

        QMetaObject::connectSlotsByName(ProgressPage);
    } // setupUi

    void retranslateUi(QWizardPage *ProgressPage)
    {
        ProgressPage->setWindowTitle(QString());
    } // retranslateUi

};

namespace Ui {
    class ProgressPage: public Ui_ProgressPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROGRESSPAGE_H
