/********************************************************************************
** Form generated from reading UI file 'addonselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADDONSELECTOR_H
#define UI_ADDONSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_AddonSelector
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *lineEdit;
    QTreeView *listView;
    QCheckBox *advancedCheckbox;

    void setupUi(QWidget *AddonSelector)
    {
        if (AddonSelector->objectName().isEmpty())
            AddonSelector->setObjectName("AddonSelector");
        AddonSelector->resize(557, 523);
        AddonSelector->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(AddonSelector);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        lineEdit = new QLineEdit(AddonSelector);
        lineEdit->setObjectName("lineEdit");
        lineEdit->setClearButtonEnabled(true);

        verticalLayout->addWidget(lineEdit);

        listView = new QTreeView(AddonSelector);
        listView->setObjectName("listView");
        listView->setMouseTracking(true);
        listView->setAlternatingRowColors(true);
        listView->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
        listView->setHorizontalScrollMode(QAbstractItemView::ScrollPerItem);
        listView->setIndentation(0);
        listView->setRootIsDecorated(false);
        listView->setItemsExpandable(false);
        listView->setSortingEnabled(true);
        listView->setHeaderHidden(true);
        listView->header()->setVisible(false);

        verticalLayout->addWidget(listView);

        advancedCheckbox = new QCheckBox(AddonSelector);
        advancedCheckbox->setObjectName("advancedCheckbox");

        verticalLayout->addWidget(advancedCheckbox);


        retranslateUi(AddonSelector);

        QMetaObject::connectSlotsByName(AddonSelector);
    } // setupUi

    void retranslateUi(QWidget *AddonSelector)
    {
        lineEdit->setPlaceholderText(fcitx::tr2fcitx("Search Addons", nullptr));
        advancedCheckbox->setText(fcitx::tr2fcitx("Show &Advanced options", nullptr));
        (void)AddonSelector;
    } // retranslateUi

};

namespace Ui {
    class AddonSelector: public Ui_AddonSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ADDONSELECTOR_H
