/****************************************************************************
** Meta object code from reading C++ file 'main.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../fcitx5-configtool/src/kcm/main.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'main.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t {};
} // unnamed namespace

template <> constexpr inline auto fcitx::kcm::FcitxModule::qt_create_metaobjectdata<qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "fcitx::kcm::FcitxModule",
        "availabilityChanged",
        "",
        "avail",
        "canRestartChanged",
        "canRestart",
        "load",
        "save",
        "defaults",
        "loadAddon",
        "saveAddon",
        "pushConfigPage",
        "title",
        "uri",
        "launchExternal",
        "saveConfig",
        "QVariant",
        "value",
        "pageNeedsSave",
        "QQuickItem*",
        "idx",
        "parseFont",
        "QFont",
        "font",
        "fontToString",
        "parseColor",
        "QColor",
        "str",
        "colorToString",
        "color",
        "grabKeyboard",
        "item",
        "ungrabKeyboard",
        "eventFilter",
        "watched",
        "QEvent*",
        "event",
        "eventToString",
        "keyCode",
        "localizedKeyString",
        "runFcitx",
        "fixLayout",
        "fixInputMethod",
        "handleAvailabilityChanged",
        "pageNeedsSaveChanged",
        "imConfig",
        "IMConfig*",
        "layoutProvider",
        "LayoutProvider*",
        "addonModel",
        "AddonProxyModel*",
        "availability"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'availabilityChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'canRestartChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'load'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'defaults'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadAddon'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAddon'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pushConfigPage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Slot 'launchExternal'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'saveConfig'
        QtMocHelpers::SlotData<void(const QString &, const QVariant &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'pageNeedsSave'
        QtMocHelpers::SlotData<QQuickItem *(int)>(18, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'parseFont'
        QtMocHelpers::SlotData<QFont(const QString &)>(21, 2, QMC::AccessPublic, 0x80000000 | 22, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'fontToString'
        QtMocHelpers::SlotData<QString(const QFont &)>(24, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'parseColor'
        QtMocHelpers::SlotData<QColor(const QString &)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'colorToString'
        QtMocHelpers::SlotData<QString(const QColor &)>(28, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 26, 29 },
        }}),
        // Slot 'grabKeyboard'
        QtMocHelpers::SlotData<void(QQuickItem *)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 31 },
        }}),
        // Slot 'ungrabKeyboard'
        QtMocHelpers::SlotData<void(QQuickItem *)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 31 },
        }}),
        // Slot 'eventFilter'
        QtMocHelpers::SlotData<bool(QObject *, QEvent *)>(33, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 34 }, { 0x80000000 | 35, 36 },
        }}),
        // Slot 'eventToString'
        QtMocHelpers::SlotData<QString(bool)>(37, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'localizedKeyString'
        QtMocHelpers::SlotData<QString(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'runFcitx'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fixLayout'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fixInputMethod'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAvailabilityChanged'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'pageNeedsSaveChanged'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'imConfig'
        QtMocHelpers::PropertyData<IMConfig*>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'layoutProvider'
        QtMocHelpers::PropertyData<LayoutProvider*>(47, 0x80000000 | 48, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'addonModel'
        QtMocHelpers::PropertyData<AddonProxyModel*>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'availability'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'canRestart'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FcitxModule, qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject fcitx::kcm::FcitxModule::staticMetaObject = { {
    QMetaObject::SuperData::link<KQuickConfigModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>.metaTypes,
    nullptr
} };

void fcitx::kcm::FcitxModule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FcitxModule *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->availabilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->canRestartChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->load(); break;
        case 3: _t->save(); break;
        case 4: _t->defaults(); break;
        case 5: _t->loadAddon(); break;
        case 6: _t->saveAddon(); break;
        case 7: _t->pushConfigPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->launchExternal((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->saveConfig((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 10: { QQuickItem* _r = _t->pageNeedsSave((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 11: { QFont _r = _t->parseFont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QFont*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->fontToString((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QColor _r = _t->parseColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 14: { QString _r = _t->colorToString((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->grabKeyboard((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 16: _t->ungrabKeyboard((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 17: { bool _r = _t->eventFilter((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { QString _r = _t->eventToString((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->localizedKeyString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->runFcitx(); break;
        case 21: _t->fixLayout(); break;
        case 22: _t->fixInputMethod(); break;
        case 23: _t->handleAvailabilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->pageNeedsSaveChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FcitxModule::*)(bool )>(_a, &FcitxModule::availabilityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FcitxModule::*)(bool )>(_a, &FcitxModule::canRestartChanged, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AddonProxyModel* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< IMConfig* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< LayoutProvider* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<IMConfig**>(_v) = _t->imConfig(); break;
        case 1: *reinterpret_cast<LayoutProvider**>(_v) = _t->layoutProvider(); break;
        case 2: *reinterpret_cast<AddonProxyModel**>(_v) = _t->addonModel(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->availability(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->canRestart(); break;
        default: break;
        }
    }
}

const QMetaObject *fcitx::kcm::FcitxModule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *fcitx::kcm::FcitxModule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5fcitx3kcm11FcitxModuleE_t>.strings))
        return static_cast<void*>(this);
    return KQuickConfigModule::qt_metacast(_clname);
}

int fcitx::kcm::FcitxModule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KQuickConfigModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void fcitx::kcm::FcitxModule::availabilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void fcitx::kcm::FcitxModule::canRestartChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
