/********************************************************************************
** Form generated from reading UI file 'fontbutton.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONTBUTTON_H
#define UI_FONTBUTTON_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>
#include <fcitxqti18nhelper.h>

QT_BEGIN_NAMESPACE

class Ui_FontButton
{
public:
    QHBoxLayout *horizontalLayout;
    QLineEdit *fontPreviewLabel;
    QPushButton *fontSelectButton;

    void setupUi(QWidget *FontButton)
    {
        if (FontButton->objectName().isEmpty())
            FontButton->setObjectName("FontButton");
        FontButton->resize(400, 186);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(FontButton->sizePolicy().hasHeightForWidth());
        FontButton->setSizePolicy(sizePolicy);
        FontButton->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(FontButton);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        fontPreviewLabel = new QLineEdit(FontButton);
        fontPreviewLabel->setObjectName("fontPreviewLabel");
        fontPreviewLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        fontPreviewLabel->setReadOnly(true);

        horizontalLayout->addWidget(fontPreviewLabel);

        fontSelectButton = new QPushButton(FontButton);
        fontSelectButton->setObjectName("fontSelectButton");

        horizontalLayout->addWidget(fontSelectButton);


        retranslateUi(FontButton);

        QMetaObject::connectSlotsByName(FontButton);
    } // setupUi

    void retranslateUi(QWidget *FontButton)
    {
        fontSelectButton->setText(fcitx::tr2fcitx("Select &Font...", nullptr));
        (void)FontButton;
    } // retranslateUi

};

namespace Ui {
    class FontButton: public Ui_FontButton {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FONTBUTTON_H
